#!/usr/bin/env python

from __future__ import annotations

import argparse
import json
import sys
import time

import boto3 as boto3
from botocore.config import Config

sqs_client = boto3.client("sqs", config=Config(retries={"mode": "standard"}))


def sqs_get_queue_url(queue_name: str) -> str:
    res = sqs_client.get_queue_url(QueueName=queue_name)
    return res["QueueUrl"]


def sqs_redrive(src_queue_url: str, dst_queue_url: str, delete_flag: bool, limit: int, wait: int):
    print(f"Redriving from {src_queue_url} to {dst_queue_url}")
    cnt = 0
    while cnt < limit:
        # receive
        res = sqs_client.receive_message(
            QueueUrl=src_queue_url,
            WaitTimeSeconds=0,
            AttributeNames=["All"],
        )
        if "Messages" not in res:
            break

        for message in res["Messages"]:
            print(json.dumps(message, ensure_ascii=False, indent=2))

            # send
            args = {
                "QueueUrl": dst_queue_url,
                "MessageBody": message["Body"],
            }
            if "MessageGroupId" in message["Attributes"]:
                # preserve MessageGroupId
                args["MessageGroupId"] = message["Attributes"]["MessageGroupId"]
            sqs_client.send_message(**args)

            # delete
            if delete_flag:
                receipt_handle = message["ReceiptHandle"]
                sqs_client.delete_message(
                    QueueUrl=src_queue_url,
                    ReceiptHandle=receipt_handle,
                )
                print("-> deleted", file=sys.stderr)

            cnt += 1
            if cnt >= limit:
                break
            if cnt % 10 == 0:
                print(f"sent {cnt}", file=sys.stderr)
            time.sleep(wait)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--src-queue-url")
    parser.add_argument("--src-queue-name")
    parser.add_argument("--dst-queue-url")
    parser.add_argument("--dst-queue-name")
    parser.add_argument("-d", "--delete", action="store_true")
    parser.add_argument("--limit", type=int, default=10)
    parser.add_argument("--wait", type=int, default=1)
    args = parser.parse_args()

    src_queue_url = args.src_queue_url
    if not args.src_queue_url:
        if not args.src_queue_name:
            print(f"ERROR: queue url or queue name required.")
            exit(1)
        src_queue_url = sqs_get_queue_url(args.src_queue_name)
    dst_queue_url = args.dst_queue_url
    if not args.dst_queue_url:
        if not args.dst_queue_name:
            print(f"ERROR: queue url or queue name required.")
            exit(1)
        dst_queue_url = sqs_get_queue_url(args.dst_queue_name)
    sqs_redrive(src_queue_url, dst_queue_url, args.delete, int(args.limit), int(args.wait))


if __name__ == "__main__":
    main()
